@@ -1,22 +1,31 @@
 from __future__ import annotations
 
 from functools import wraps, partial
+from dataclasses import asdict
 from pathlib import Path
 
 from alphafold3_pytorch.alphafold3 import Alphafold3
-from alphafold3_pytorch.attention import pad_at_dim
+from alphafold3_pytorch.attention import pad_at_dim, pad_or_slice_to
 
 from typing import TypedDict, List, Callable
 
-from alphafold3_pytorch.typing import (
+from alphafold3_pytorch.tensor_typing import (
     typecheck,
-    beartype_isinstance,
     Int, Bool, Float
 )
 
+from alphafold3_pytorch.attention import (
+    full_pairwise_repr_to_windowed,
+    full_attn_bias_to_windowed
+)
+
 from alphafold3_pytorch.inputs import (
     AtomInput,
-    INPUT_TO_ATOM_TRANSFORM
+    BatchedAtomInput,
+    Alphafold3Input,
+    PDBInput,
+    maybe_transform_to_atom_inputs,
+    alphafold3_input_to_molecule_input
 )
 
 import torch
@@ -68,12 +77,13 @@ def accum_dict(
 # dataloader and collation fn
 
 @typecheck
-def collate_af3_inputs(
+def collate_inputs_to_batched_atom_input(
     inputs: List,
     int_pad_value = -1,
+    atoms_per_window: int | None = None,
     map_input_fn: Callable | None = None
 
-) -> AtomInput:
+) -> BatchedAtomInput:
 
     if exists(map_input_fn):
         inputs = [map_input_fn(i) for i in inputs]
@@ -81,24 +91,30 @@ def collate_af3_inputs(
     # go through all the inputs
     # and for any that is not AtomInput, try to transform it with the registered input type to corresponding registered function
 
-    atom_inputs = []
+    atom_inputs = maybe_transform_to_atom_inputs(inputs)
 
-    for i in inputs:
-        if beartype_isinstance(i, AtomInput):
-            atom_inputs.append(i)
-            continue
+    # take care of windowing the atompair_inputs and atompair_ids if they are not windowed already
 
-        maybe_to_atom_fn = INPUT_TO_ATOM_TRANSFORM.get(type(i), None)
+    if exists(atoms_per_window):
+        for atom_input in atom_inputs:
+            atompair_inputs = atom_input.atompair_inputs
+            atompair_ids = atom_input.atompair_ids
 
-        if not exists(maybe_to_atom_fn):
-            raise TypeError(f'invalid input type {type(i)} being passed into Trainer that is not converted to AtomInput correctly')
+            atompair_inputs_is_windowed = atompair_inputs.ndim == 4
 
-        atom_inputs.append(maybe_to_atom_fn(i))
+            if not atompair_inputs_is_windowed:
+                atom_input.atompair_inputs = full_pairwise_repr_to_windowed(atompair_inputs, window_size = atoms_per_window)
+
+            if exists(atompair_ids):
+                atompair_ids_is_windowed = atompair_ids.ndim == 3
+
+                if not atompair_ids_is_windowed:
+                    atom_input.atompair_ids = full_attn_bias_to_windowed(atompair_ids, window_size = atoms_per_window)
 
     # separate input dictionary into keys and values
 
-    keys = atom_inputs[0].keys()
-    atom_inputs = [i.values() for i in atom_inputs]
+    keys = atom_inputs[0].dict().keys()
+    atom_inputs = [i.dict().values() for i in atom_inputs]
 
     outputs = []
 
@@ -156,18 +172,55 @@ def collate_af3_inputs(
 
         outputs.append(stacked)
 
+    # batched atom input dictionary
+
+    batched_atom_input_dict = dict(tuple(zip(keys, outputs)))
+
+    # just ensure output_atompos_indices has full atom_seq_len manually for now
+
+    output_atompos_indices = batched_atom_input_dict.get('output_atompos_indices', None)
+
+    if exists(output_atompos_indices):
+        atom_seq_len = batched_atom_input_dict['atom_inputs'].shape[-2]
+        batched_atom_input_dict.update(output_atompos_indices = pad_or_slice_to(output_atompos_indices, atom_seq_len, dim = -1, pad_value = -1))
+
     # reconstitute dictionary
 
-    batched_atom_inputs = AtomInput(tuple(zip(keys, outputs)))
+    batched_atom_inputs = BatchedAtomInput(**batched_atom_input_dict)
     return batched_atom_inputs
 
+@typecheck
+def alphafold3_inputs_to_batched_atom_input(
+    inp: Alphafold3Input | List[Alphafold3Input],
+    **collate_kwargs
+) -> BatchedAtomInput:
+
+    if isinstance(inp, Alphafold3Input):
+        inp = [inp]
+
+    atom_inputs = maybe_transform_to_atom_inputs(inp)
+    return collate_inputs_to_batched_atom_input(atom_inputs, **collate_kwargs)
+
+@typecheck
+def pdb_inputs_to_batched_atom_input(
+    inp: PDBInput | List[PDBInput],
+    **collate_kwargs
+) -> BatchedAtomInput:
+
+    if isinstance(inp, PDBInput):
+        inp = [inp]
+
+    atom_inputs = maybe_transform_to_atom_inputs(inp)
+    return collate_inputs_to_batched_atom_input(atom_inputs, **collate_kwargs)
+
 @typecheck
 def DataLoader(
     *args,
+    atoms_per_window: int | None = None,
     map_input_fn: Callable | None = None,
     **kwargs
 ):
-    collate_fn = collate_af3_inputs
+    collate_fn = partial(collate_inputs_to_batched_atom_input, atoms_per_window = atoms_per_window)
 
     if exists(map_input_fn):
         collate_fn = partial(collate_fn, map_input_fn = map_input_fn)
@@ -222,7 +275,9 @@ class Trainer:
         checkpoint_folder: str = './checkpoints',
         overwrite_checkpoints: bool = False,
         fabric_kwargs: dict = dict(),
-        ema_kwargs: dict = dict()
+        ema_kwargs: dict = dict(
+            use_foreach = True
+        )
     ):
         super().__init__()
 
@@ -259,7 +314,7 @@ class Trainer:
 
         # if map dataset function given, curry into DataLoader
 
-        DataLoader_ = DataLoader
+        DataLoader_ = partial(DataLoader, atoms_per_window = model.atoms_per_window)
 
         if exists(map_dataset_input_fn):
             DataLoader_ = partial(DataLoader_, map_input_fn = map_dataset_input_fn)
@@ -494,7 +549,7 @@ class Trainer:
                     # model forwards
 
                     loss, loss_breakdown = self.model(
-                        **inputs,
+                        **inputs.dict(),
                         return_loss_breakdown = True
                     )
 
@@ -554,11 +609,11 @@ class Trainer:
 
                     for valid_batch in self.valid_dataloader:
                         valid_loss, loss_breakdown = self.ema_model(
-                            **valid_batch,
+                            **valid_batch.dict(),
                             return_loss_breakdown = True
                         )
 
-                        valid_batch_size = valid_batch.get('atom_inputs').shape[0]
+                        valid_batch_size = valid_batch.atom_inputs.shape[0]
                         scale = valid_batch_size / self.valid_dataset_size
 
                         total_valid_loss += valid_loss.item() * scale
@@ -592,11 +647,11 @@ class Trainer:
 
                 for test_batch in self.test_dataloader:
                     test_loss, loss_breakdown = self.ema_model(
-                        **test_batch,
+                        **test_batch.dict(),
                         return_loss_breakdown = True
                     )
 
-                    test_batch_size = test_batch.get('atom_inputs').shape[0]
+                    test_batch_size = test_batch.atom_inputs.shape[0]
                     scale = test_batch_size / self.test_dataset_size
 
                     total_test_loss += test_loss.item() * scale
