@@ -150,7 +150,7 @@ def test_compute_alignment_error():
     error_fn = ComputeAlignmentError()
     alignment_errors = error_fn(pred_coords, pred_coords, pred_frames, pred_frames)
 
-    assert alignment_errors.shape == (2, 100)
+    assert alignment_errors.shape == (2, 100, 100)
     assert (alignment_errors.mean(-1) < 1e-3).all()
 
 def test_centre_random_augmentation():
@@ -163,8 +163,10 @@ def test_centre_random_augmentation():
 
 
 @pytest.mark.parametrize('recurrent_depth', (1, 2))
+@pytest.mark.parametrize('enable_attn_softclamp', (True, False))
 def test_pairformer(
-    recurrent_depth
+    recurrent_depth,
+    enable_attn_softclamp
 ):
     single = torch.randn(2, 16, 384)
     pairwise = torch.randn(2, 16, 16, 128)
@@ -173,7 +175,10 @@ def test_pairformer(
     pairformer = PairformerStack(
         depth = 4,
         num_register_tokens = 4,
-        recurrent_depth = recurrent_depth
+        recurrent_depth = recurrent_depth,
+        pair_bias_attn_kwargs = dict(
+            enable_attn_softclamp = enable_attn_softclamp
+        )
     )
 
     single_out, pairwise_out = pairformer(
@@ -206,7 +211,8 @@ def test_msa_module():
     assert pairwise.shape == pairwise_out.shape
 
 @pytest.mark.parametrize('use_linear_attn', (False, True))
-def test_diffusion_transformer(use_linear_attn):
+@pytest.mark.parametrize('use_colt5_attn', (False, True))
+def test_diffusion_transformer(use_linear_attn, use_colt5_attn):
 
     single = torch.randn(2, 16, 384)
     pairwise = torch.randn(2, 16, 16, 128)
@@ -215,7 +221,8 @@ def test_diffusion_transformer(use_linear_attn):
     diffusion_transformer = DiffusionTransformer(
         depth = 2,
         heads = 16,
-        use_linear_attn = use_linear_attn
+        use_linear_attn = use_linear_attn,
+        use_colt5_attn = use_colt5_attn
     )
 
     single_out = diffusion_transformer(
@@ -328,7 +335,7 @@ def test_diffusion_module():
     assert sampled_atom_pos.shape == noised_atom_pos.shape
     
 def test_relative_position_encoding():
-    additional_molecule_feats = torch.randn(8, 100, 10)
+    additional_molecule_feats = torch.randint(0, 2, (8, 100, 5))
 
     embedder = RelativePositionEncoding()
 
@@ -385,7 +392,8 @@ def test_input_embedder():
     atompair_inputs = torch.randn(2, atom_seq_len, atom_seq_len, 5)
 
     atom_mask = torch.ones((2, atom_seq_len)).bool()
-    additional_molecule_feats = torch.randn(2, 16, 10)
+    additional_token_feats = torch.randn(2, 16, 2)
+    molecule_ids = torch.randint(0, 32, (2, 16))
 
     embedder = InputFeatureEmbedder(
         dim_atom_inputs = 77,
@@ -396,7 +404,8 @@ def test_input_embedder():
         atom_mask = atom_mask,
         atompair_inputs = atompair_inputs,
         molecule_atom_lens = molecule_atom_lens,
-        additional_molecule_feats = additional_molecule_feats
+        molecule_ids = molecule_ids,
+        additional_token_feats = additional_token_feats
     )
 
 def test_distogram_head():
@@ -408,9 +417,13 @@ def test_distogram_head():
 
 @pytest.mark.parametrize('window_atompair_inputs', (True, False))
 @pytest.mark.parametrize('stochastic_frame_average', (True, False))
+@pytest.mark.parametrize('atom_transformer_intramolecular_attn', (True, False))
+@pytest.mark.parametrize('num_molecule_mods', (0, 5))
 def test_alphafold3(
     window_atompair_inputs: bool,
-    stochastic_frame_average: bool
+    stochastic_frame_average: bool,
+    atom_transformer_intramolecular_attn: bool,
+    num_molecule_mods: int
 ):
     seq_len = 16
     atoms_per_window = 27
@@ -418,7 +431,7 @@ def test_alphafold3(
     molecule_atom_lens = torch.randint(1, 3, (2, seq_len))
     atom_seq_len = molecule_atom_lens.sum(dim = -1).amax()
 
-    token_bond = torch.randint(0, 2, (2, seq_len, seq_len)).bool()
+    token_bonds = torch.randint(0, 2, (2, seq_len, seq_len)).bool()
 
     atom_inputs = torch.randn(2, atom_seq_len, 77)
 
@@ -427,7 +440,19 @@ def test_alphafold3(
     if window_atompair_inputs:
         atompair_inputs = full_pairwise_repr_to_windowed(atompair_inputs, window_size = atoms_per_window)
 
-    additional_molecule_feats = torch.randn(2, seq_len, 10)
+    additional_molecule_feats = torch.randint(0, 2, (2, seq_len, 5))
+    additional_token_feats = torch.randn(2, 16, 2)
+    is_molecule_types = torch.randint(0, 2, (2, seq_len, 4)).bool()
+    molecule_ids = torch.randint(0, 32, (2, seq_len))
+
+    is_molecule_mod = None
+    if num_molecule_mods > 0:
+        is_molecule_mod = torch.zeros(2, seq_len, num_molecule_mods).uniform_(0, 1) < 0.1
+
+    atom_parent_ids = None
+
+    if atom_transformer_intramolecular_attn:
+        atom_parent_ids = torch.ones(2, atom_seq_len).long()
 
     template_feats = torch.randn(2, 2, seq_len, seq_len, 44)
     template_mask = torch.ones((2, 2)).bool()
@@ -448,6 +473,7 @@ def test_alphafold3(
         atoms_per_window = atoms_per_window,
         dim_template_feats = 44,
         num_dist_bins = 38,
+        num_molecule_mods = num_molecule_mods,
         confidence_head_kwargs = dict(
             pairformer_depth = 1
         ),
@@ -471,10 +497,15 @@ def test_alphafold3(
     loss, breakdown = alphafold3(
         num_recycling_steps = 2,
         atom_inputs = atom_inputs,
+        molecule_ids = molecule_ids,
         molecule_atom_lens = molecule_atom_lens,
+        atom_parent_ids = atom_parent_ids,
         atompair_inputs = atompair_inputs,
+        is_molecule_types = is_molecule_types,
+        is_molecule_mod = is_molecule_mod,
         additional_molecule_feats = additional_molecule_feats,
-        token_bond = token_bond,
+        additional_token_feats = additional_token_feats,
+        token_bonds = token_bonds,
         msa = msa,
         msa_mask = msa_mask,
         templates = template_feats,
@@ -494,9 +525,13 @@ def test_alphafold3(
     sampled_atom_pos = alphafold3(
         num_sample_steps = 16,
         atom_inputs = atom_inputs,
+        molecule_ids = molecule_ids,
         molecule_atom_lens = molecule_atom_lens,
+        is_molecule_mod = is_molecule_mod,
         atompair_inputs = atompair_inputs,
+        is_molecule_types = is_molecule_types,
         additional_molecule_feats = additional_molecule_feats,
+        additional_token_feats = additional_token_feats,
         msa = msa,
         templates = template_feats,
         template_mask = template_mask,
@@ -511,7 +546,10 @@ def test_alphafold3_without_msa_and_templates():
 
     atom_inputs = torch.randn(2, atom_seq_len, 77)
     atompair_inputs = torch.randn(2, atom_seq_len, atom_seq_len, 5)
-    additional_molecule_feats = torch.randn(2, seq_len, 10)
+    additional_molecule_feats = torch.randint(0, 2, (2, seq_len, 5))
+    additional_token_feats = torch.randn(2, seq_len, 2)
+    is_molecule_types = torch.randint(0, 2, (2, seq_len, 4)).bool()
+    molecule_ids = torch.randint(0, 32, (2, seq_len))
 
     atom_pos = torch.randn(2, atom_seq_len, 3)
     molecule_atom_indices = molecule_atom_lens - 1
@@ -548,9 +586,12 @@ def test_alphafold3_without_msa_and_templates():
     loss, breakdown = alphafold3(
         num_recycling_steps = 2,
         atom_inputs = atom_inputs,
+        molecule_ids = molecule_ids,
         molecule_atom_lens = molecule_atom_lens,
         atompair_inputs = atompair_inputs,
+        is_molecule_types = is_molecule_types,
         additional_molecule_feats = additional_molecule_feats,
+        additional_token_feats = additional_token_feats,
         atom_pos = atom_pos,
         molecule_atom_indices = molecule_atom_indices,
         distance_labels = distance_labels,
@@ -570,7 +611,10 @@ def test_alphafold3_force_return_loss():
 
     atom_inputs = torch.randn(2, atom_seq_len, 77)
     atompair_inputs = torch.randn(2, atom_seq_len, atom_seq_len, 5)
-    additional_molecule_feats = torch.randn(2, seq_len, 10)
+    additional_molecule_feats = torch.randint(0, 2, (2, seq_len, 5))
+    additional_token_feats = torch.randn(2, seq_len, 2)
+    is_molecule_types = torch.randint(0, 2, (2, seq_len, 4)).bool()
+    molecule_ids = torch.randint(0, 32, (2, seq_len))
 
     atom_pos = torch.randn(2, atom_seq_len, 3)
     molecule_atom_indices = molecule_atom_lens - 1
@@ -607,9 +651,12 @@ def test_alphafold3_force_return_loss():
     sampled_atom_pos = alphafold3(
         num_recycling_steps = 2,
         atom_inputs = atom_inputs,
+        molecule_ids = molecule_ids,
         molecule_atom_lens = molecule_atom_lens,
         atompair_inputs = atompair_inputs,
+        is_molecule_types = is_molecule_types,
         additional_molecule_feats = additional_molecule_feats,
+        additional_token_feats = additional_token_feats,
         atom_pos = atom_pos,
         molecule_atom_indices = molecule_atom_indices,
         distance_labels = distance_labels,
@@ -626,9 +673,12 @@ def test_alphafold3_force_return_loss():
     loss, _ = alphafold3(
         num_recycling_steps = 2,
         atom_inputs = atom_inputs,
+        molecule_ids = molecule_ids,
         molecule_atom_lens = molecule_atom_lens,
         atompair_inputs = atompair_inputs,
+        is_molecule_types = is_molecule_types,
         additional_molecule_feats = additional_molecule_feats,
+        additional_token_feats = additional_token_feats,
         return_loss_breakdown = True,
         return_loss = True # force returning loss even if no labels given
     )
@@ -656,7 +706,10 @@ def test_alphafold3_with_atom_and_bond_embeddings():
     atom_inputs = torch.randn(2, atom_seq_len, 77)
     atompair_inputs = torch.randn(2, atom_seq_len, atom_seq_len, 5)
 
-    additional_molecule_feats = torch.randn(2, seq_len, 10)
+    additional_molecule_feats = torch.randint(0, 2, (2, seq_len, 5))
+    additional_token_feats = torch.randn(2, seq_len, 2)
+    is_molecule_types = torch.randint(0, 2, (2, seq_len, 4)).bool()
+    molecule_ids = torch.randint(0, 32, (2, seq_len))
 
     template_feats = torch.randn(2, 2, seq_len, seq_len, 44)
     template_mask = torch.ones((2, 2)).bool()
@@ -683,8 +736,11 @@ def test_alphafold3_with_atom_and_bond_embeddings():
         atompair_ids = atompair_ids,
         atom_inputs = atom_inputs,
         atompair_inputs = atompair_inputs,
+        molecule_ids = molecule_ids,
         molecule_atom_lens = molecule_atom_lens,
+        is_molecule_types = is_molecule_types,
         additional_molecule_feats = additional_molecule_feats,
+        additional_token_feats = additional_token_feats,
         msa = msa,
         msa_mask = msa_mask,
         templates = template_feats,
